#ifndef __FIELD_GROUP__
#define __FIELD_GROUP__


using XMLIOSERVER::XML::THashAttributes;

namespace XMLIOSERVER
{   
   class FieldGroup : public GroupTemplate<Field, FieldAttribut>
   {
      public:
      
         FieldGroup(void) : GroupTemplate<Field, FieldAttribut>()
         {/* Ne rien faire de plus */}               
         FieldGroup(const string& _id) : GroupTemplate<Field, FieldAttribut>(_id)
         {/* Ne rien faire de plus */}
         
         void setAttributes(const THashAttributes& _attr)
         {
            for (THashAttributes::ConstIterator it = _attr.begin(); it != _attr.end(); it++)
               if ((*it).first.compare(string("id"))!= 0 and (*it).first.compare(string("src"))!=0)
               // (Au dessus) Non prise en compte de l'identifiant et de l'inclusion de fichiers externes lors de l'affectation des attributs.
                  this->setSAttribut((*it).first, (*it).second);
            
            return;
         }
                 
         const char* getName(void) const {return ("FieldGroup"); }  
         
         friend ostream& operator<< (ostream& out, const FieldGroup& c) 
         {            
            out << "<" << c.getName();
            if(c.hasId()) out << " id=\"" <<  c.getId() << "\"";
            for(unsigned int i = 0; i < c.attrList.getVectorSize(); i++) out << *c.attrList.getVector()[i];
            out << ">" << std::endl;
            
            for(unsigned int i = 0; i < c.groupList.getVector().size() ; i++)
               out << *((FieldGroup*)c.groupList.getVector()[i])  << std::endl;
               
            for(unsigned int i = 0; i < c.childList.getVector().size() ; i++)
               out << *(c.childList.getVector()[i]) << std::endl;
               
            out << "</" << c.getName()<< ">";
            return (out);
         } 
                          
         void addAttributes(const StrHashMap<BaseAttribut>& _pattr)
         {          
            StrHashMap<BaseAttribut>& _lattr = getAttributList();
            
            //_pattr contient les attributs du parent, _lattr les attributs locaux.
            
            for(unsigned int i = 0; i < _lattr.getVectorSize(); i++)
            {
               if(_lattr.getVector()[i]->_hasValue() or !_pattr.getVector()[i]->_hasValue()) continue;
               _lattr.getVector()[i]->assignValue(_pattr.getVector()[i]);
            }
         }
         
         void resolveDescInheritance(StrHashMap<BaseAttribut>* _pattr = 0)
         {
            const vector<Field*>&  childvect = childList.getVector();
            const vector<GroupTemplate<Field, FieldAttribut>*>& groupvect = groupList.getVector();
            
            // On complète les propres attributs du groupe.
            if (_pattr!= NULL) addAttributes(*_pattr);
            
            for(unsigned int i = 0; i < childvect.size() ; i++)
            // on complète les attributs des champs enfants
               childvect[i] -> addAttributes(this->attrList);
               
            for(unsigned int i = 0; i < groupvect.size() ; i++)
            // on complète les attributs des groupes de champs enfants
               ((FieldGroup*)groupvect[i]) -> resolveDescInheritance(&this->attrList);
         }
         
         void parse (XMLNode& _node)
         {
            string name = _node.getElementName();            
            THashAttributes attributes;

            /// PARSING GESTION DES ATTRIBUTS ///
            _node.getAttributes(attributes);  
            this->setAttributes(attributes);
            
            if (attributes.end() != attributes.find("src"))
            { // Si une demande d'inclusion de fichier est trouvé.
               XMLNode _node_inc = getNodeIncludedFile(attributes["src"], name);
               parse (_node_inc);
            }
            
            attributes.clear();
               
            /// PARSING POUR GESION DES ENFANTS
            if (!(_node.goToChildElement()))
               WARNING("Le groupe de champ ne contient pas d'enfant !");
            else
            {
               //////////////////////////////////////
               do { // Parcours des contexts pour traitement. 
                            
                  string name = _node.getElementName();
                  attributes.clear();
                  _node.getAttributes(attributes);  
                                 
                  if (name.compare("field_group") == 0)
                  { // Parsing pour les groupes de champs
                  
                     FieldGroup* fgroup = NULL;
                        
                     if (attributes.end() != attributes.find("id"))
                     {// Si l'identifiant est défini.
                        if (FieldGroup::HasObject(attributes["id"]))
                           WARNING("Dans le context actuel, un groupe de champ du même nom existe déjà, le second fera référence au premier par défaut !"); // TODO TODO
                        fgroup = (FieldGroup*)(&createGroup(attributes["id"]));
                        fgroup->parse(_node);
                     }
                     else
                     {// Si l'identifiant n'est pas défini.
                        fgroup = (FieldGroup*)(&createGroup());
                        fgroup->parse(_node);
                     }
                        
                     continue;
                        
                  }
                  else if (name.compare("field") == 0)
                  { // Parsing pour les champs
                  
                     Field* field = NULL;
                  
                     if (attributes.end() != attributes.find("id"))
                     {// Si l'identifiant est défini.
                        if (Field::HasObject(attributes["id"]))
                           WARNING("Dans le context actuel, un champ du même nom existe déjà, le second fera référence au premier par défaut !");  // TODO TODO
                        field = (Field*)(&createChild(attributes["id"]));
                        field->parse(_node);
                     }
                     else
                     {// Si l'identifiant n'est pas défini.
                        field = (Field*)(&createChild());
                        field->parse(_node);
                     }         
                    
                     continue;
                  } 
                  else
                     WARNING("Un groupe de champs ne peut contenir qu'un champ ou un autre groupe de champs !");
                     
               } while (_node.goToNextElement());
               //////////////////////////////////////
               _node.goToParentElement(); // Retour au parent  
            }
                                       
            return;
         }
         
         virtual ~FieldGroup(void) 
         {/* Ne rien faire de plus */ }
                 
   }; // class FieldGroup
     
   typedef FieldGroup FieldDefinition ;
   
}; // namespace XMLIOSERVER
   
#endif // __FIELD_GROUP__

