#ifndef __XMLIO_XML_PARSER__
#define __XMLIO_XML_PARSER__ 

#include <stdlib.h>

// Utilisation de la STL
using std::string;

namespace XMLIOSERVER
{
   namespace XML
   {          
      using XMLIOSERVER::Context;
      
      class XMLParser
      {
         public : 
         
            static void Parse(XMLNode& _node)
            {
               THashAttributes attributes;
               
               _node.goToChildElement(); // On se place sur le premier élément enfant du noeud racine.
               atexit (&XMLParser::FreeMemory); // Pour supprimer la mémoire liée à l'allocation dynamique en fin d'exécution du programme.  

                              
               do {                
                 // Traitement de l'identifiant
                  _node.getAttributes(attributes);
                            
                  if (attributes.end() == attributes.find("id"))
                  { WARNING("Le context ne sera pas traité car il n'est pas identifié !"); continue; }
                  
                  if( Context::HasObject(attributes["id"]))
                  { WARNING("Le context ne sera pas traité car il existe déjà un autre context possédant le même nom !"); continue; }
                  
                  Context::SetCurrentContext(attributes["id"]);
                  Context& context = Context::CreateObject(attributes["id"]);
                  
                  context.parse(_node);
                  attributes.clear();
                  
               } while (_node.goToNextElement());               
            }
            
            static void ResolveInheritance(void)
            {
               HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
               
               for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
               {
                  string& id = (*it).first;
                  StrHashMap<Context>& sc = (*it).second;
                  
                  // Résolution des héritages des descendands (càd des héritages de groupes) pour chacun des contextes.
                  sc[id]->resolveDescInheritance();
               }               
            }
            
            static void ShowTree(ostream& os = std::clog)
            {
               clog <<  "<?xml version=\"1.0\"?>" << std::endl << "<Simulation>" << std::endl;
               HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
               
               for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
               {
                  string& id = (*it).first;
                  StrHashMap<Context>& sc = (*it).second;
                  
                  // On sort chacun des contextes successivement.
                  clog << *sc[id] << std::endl;
               } 
               clog << "</Simulation>" << std::endl  ;
            }
            
            static void FreeMemory(void)
            {
               HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
               
               for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
               {
                  string& id = (*it).first;
                  StrHashMap<Context>& sc = (*it).second;
                  delete sc[id];
               }
            }
                 
      }; // class XMLParser     
   }; // namespace XML

};// namespace XMLIOSERVER


#endif // __XMLIO_XML_PARSER__ 
